@@ -1,15 +1,14 @@
 from __future__ import annotations
 
+import asyncio
+import inspect
 import logging
-from contextlib import ExitStack
-from contextvars import ContextVar
+from contextlib import AsyncExitStack, ExitStack
 from typing import Dict, List, Optional, Union
 
-from fastapi import FastAPI
-from sqlalchemy import MetaData, create_engine
-from sqlalchemy.engine import Engine
+from curio.meta import from_coroutine
 from sqlalchemy.engine.url import URL
-from sqlalchemy.orm import DeclarativeMeta, Session, sessionmaker
+from sqlalchemy.orm import sessionmaker
 from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
 from starlette.requests import Request
 from starlette.types import ASGIApp
@@ -17,6 +16,7 @@ from starlette.types import ASGIApp
 from .exceptions import SQLAlchemyType
 from .extensions import SQLAlchemy
 from .extensions import db as db_
+from .extensions import reset_session, start_session
 
 
 class DBStateMap:
@@ -34,6 +34,14 @@ class DBStateMap:
             raise ValueError("DBStateMap is already initialized")
 
 
+def is_async():
+    try:
+        asyncio.get_running_loop()
+        return True
+    except RuntimeError:
+        return False
+
+
 class DBSessionMiddleware(BaseHTTPMiddleware):
     def __init__(
         self,
@@ -57,13 +65,33 @@ class DBSessionMiddleware(BaseHTTPMiddleware):
             ]
         elif type(db) == list:
             self.dbs = db
-        print(app)
         for db in self.dbs:
-            db.init()
             db.create_all()
 
-    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
-        with ExitStack() as stack:
-            contexts = [stack.enter_context(ctx()) for ctx in self.dbs]
-            response = await call_next(request)
+    def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        req_async = False
+        for route in self.app.app.app.routes:
+            if route.path == request.scope["path"]:
+                req_async = inspect.iscoroutinefunction(route.endpoint)
+
+        async def dispatch_inner():
+            token = start_session()
+            async with AsyncExitStack() as async_stack:
+                with ExitStack() as sync_stack:
+                    contexts = [
+                        await async_stack.enter_async_context(ctx())
+                        if ctx.async_
+                        else sync_stack.enter_context(ctx())
+                        for ctx in self.dbs
+                    ]
+                    response = await call_next(request)
+            reset_session(token)
+            return response
+
+        if req_async:
+            return dispatch_inner()
+        else:
+            with ExitStack() as stack:
+                contexts = [stack.enter_context(ctx()) for ctx in self.dbs]
+                response = call_next(request)
         return response
